%    Alg10.m  
function [x,J,fx,Dinv,msg,dual,j,alg] = ...
                          Alg10(c,C,n,m,r,tol,x,J,Dinv,A,b)
%   Alg10 implements Algorithm 10 to find a strong local
%   minimizer (if one exists).

%   What follows is Algorithm 4 (down to the cut line.  
%   When Alg4 determines 'Optimal solution obtained',
%   we interpret this as a Karush-Kuhn-Tucker point has
%   been located and we have to transfer to Step 4 for 
%   further testing.
%   
%  Initialization
alg = 'Algorithm 10';
fx = c'*x + 0.5*( x'*C*x);
g = c + C*x;
next = 'Step1';
for j=0:10000;
    if strcmp(next,'Step1')
        [next,k,s,msg,dual,big] = ...
                      Step1(J,next,g,Dinv,n,m,r,tol);
        if strcmp(msg,'Optimal solution obtained')
           lista = find( J >= 1 );
           if length(lista) == n
               msg = 'strong local min {extreme point}'
               return
           end
           next = 'Step4';
        end    
    end
    
    if strcmp(next,'Step2')
        [next,sigma,sigtil,sighat,ell,stCs,msg] = ...
                     Step2(J,m,A,b,C,s,tol,big,x);
        if strcmp(msg,'Unbounded from below');
            return
        end    
    end    
    
    if strcmp(next,'Step3')
                                         %  Step 3: Update
        x = x - sigma*s;
        g = c + C*x;
        fx = c'*x + 0.5*( x'*C*x);
        if sigtil <= sighat
            next = 'Step3p1';
        else
            next = 'Step3p2';
        end
    end    
    if strcmp(next,'Step3p1')
                                         %  Step 3.1
        d = stCs^(-.5) * C * s;
        [Dinv] = Phi1(Dinv,d,k,J,n);     % k is from Step 1
        J(k) = -1;
        next = 'Step1';                  % end of Step 3.1               
    end
        
    if strcmp(next,'Step3p2')
                                         % Step 3.2
        list = find( J == -1 );
        big = -1.;
        for i=1:length(list)
            dotprod = abs( A(ell,:)*Dinv(:,list(i)) );
            if dotprod > big
                big = dotprod;
            end
        end
        if big >= 1.e-6
            next = 'Step3p3';
        end
        if strcmp(next,'Step3p2')
            [Dinv] =  Phi1(Dinv,A(ell,:)',k,J,n);
            J(k) = ell;
            next = 'Step1';
        end
    end
        
    if strcmp(next,'Step3p3')
                                         %  Step 3.3
        big = -1;
        nu = 0;
        for i=1:length(list)
            dotprod = abs( A(ell,:)*Dinv(:,list(i)) ) ;
            if dotprod > big
                big = dotprod;
                nu = list(i);
            end
        end
        [Dhatinv] = Phi2(Dinv,A(ell,:)',nu,J,n);
        [Dinv] = Phi1(Dhatinv,A(ell,:)',nu,J,n);
        J(nu) = ell;
        J(k) = 0;
        next = 'Step1';
    end
    

% ---------------- end of Alg 4 --------------------------    

    if strcmp(next,'Step4')
        next = 'Step4p1'
    end    
    if strcmp(next,'Step4p1')                % Step 4.1
        list = find(J == 0);
        if length(list) >= 1;
            for i=1:length(list)
                s = Dinv(:,list(i))
                stCs = s'*C*s
                if stCs >= tol
                    d = stCs^(-.5) * C * s;
                    [Dinv] = Phi1(Dinv,d,list(i),J,n);
                    J(list(i)) = -1;
                end
            end
        end
        next = 'Step4p2'
    end
    if strcmp(next,'Step4p2')                
                                                     % Step 4.2 
        number = 0
        list = find( J==0 )
        if length(list) >= 1
            for i=1:length(list)
                test = Dinv(:,list(i))
                stCs = test'*C*test
                if stCs <= -tol
                    number = number + 1
                    k = list(i)
                    s = test
                end
            end
        end
        if number >= 1
            % here we have s = Dinv(:,k) and s'Cs < 0
            next = 'Step2'
        else
            next = 'Step4p3'
        end
    end
    if strcmp(next,'Step4p3')    
    
    %  test to see if c_{ij}'*C*c_{ij} = 0 for all
    %  i with alpha_{ij} = 0  (which should be true)
        for i=1:n
            if J(i) == 0
                check = Dinv(:,i)' * C * Dinv(:,i)
            end
        end
        
                                           % Step 4.3
        rho = 0
        k = 0
        for i=1:n
            for j=i+1:n
                if J(i) == 0 & J(j) == 0
                    check1 =  Dinv(:,i)' * C * Dinv(:,j)
                    if abs(check1) >= tol
                        rho = i
                        k = j
                    end
                end
            end
        end
        if k >= 1 & rho >= 1
            check2 =  Dinv(:,rho)' * C * Dinv(:,k)
            if check2 < 0
                s = Dinv(:,rho) + Dinv(:,k)
            else
                s = Dinv(:,rho) - Dinv(:,k)
            end
            next = 'Step2'
        else                    % optimal solution
            list = find(J ~= 0)
            if length(list) == n
                msg = 'Strong local min'
                return
            end
            if length(list) == n-1
                msg = 'Weak local min'
                return
            end
        end
    end
end    % iteration loop for j



