%  Alg5.m
function [x,J,fx,Dinv,msg,dual,iter,alg] = ...
                            Alg5(c,C,n,m,r,tol,x,J,Dinv,A,b)
%   implements QP Algorithm 5, which is a dual 
%         QP Algorithm.  
%  Initialization
alg = 'Algorithm 5';
fx = c'*x + 0.5*( x'*C*x);
g = c + C*x;
dfeas = 1;
for i=1:n
   if J(i) <= 0
       if abs(g'*Dinv(:,i)) >= tol
           dfeas = 0;
        end
   end    
   if J(i) >= 1 & J(i) <= m
       if g'*Dinv(:,i) >= tol            
           dfeas = 0;
       end
   end
end   %end of for loop

if dfeas == 0;
    % initial point not dual feasible.
    msg = 'initial point not dual feasible.'
    return
end

for iter=0:10000;
    p11 = 0;
   for i=1:n
       if J(i) > 0
           if abs( A(J(i),:)*x - b(J(i)) ) > tol
               p11 =  1;
           end
       end
   end
   if p11 == 1
       next = 'Step1p1';
   else
       next = 'Step1p2';
   end
   if strcmp(next,'Step1p1')
                                % Step 1.1:
       big = - inf;
       index  = 0;
       for i=1:n
           if J(i) > 0
               test = abs( A(J(i),:)*x - b(J(i)) );
               if test > big
                   big = test;
                   index = i;
               end
           end
       end    %end of for loop
       nu = index;
       ell = J(nu);
       next = 'Step2';                                        
   end    % end of Step 1.1
   
   if strcmp(next,'Step1p2')
                                % Step 1.2:
       big = - inf;                             
       ell = 0;
       for i=1:m+r
           test = A(i,:)*x - b(i);
           if i > m;
               test = abs(test);
           end
           if test > big
               big = test;
               ell = i;
           end
       end   %end of for loop
       if big <= tol
           %  Opimal Solution Obtained
           msg = 'Optimal solution obtained';
           [dual] = GetDual(Dinv,J,g,n,m,r);
           return
       else
           next = 'Step1p3';
       end    
   end    % end of Step 1.2       
   if strcmp(next,'Step1p3')
                                % Step 1.3 
       p13 = 0;
       for i=1:n
           if J(i) <= 0 & abs(A(ell,:)*Dinv(:,i)) >= tol
               p13 =  1;
           end    
       end   %end of for loop
       if p13 == 1
           next = 'Step1p4';
       else
           nu = 0;
           minratio = inf;
           infeas = 1;
           for i=1:n
               if 1 <= J(i) & J(i) <= m
                   lambda = A(ell,:)*Dinv(:,i);
                   if lambda > 0
                       infeas = 0;
                   end
                   if lambda > 0
                       vee = - g'*Dinv(:,i);
                       ratio = vee/lambda;
                       if ratio < minratio
                           minratio = ratio;
                           nu = i;
                       end
                   end
               end
           end     %end of for loop
           if infeas == 1;
              msg = 'No feasible solution'
           return;
           end
           Dhatinv = Dinv;
           next = 'Step4p1';    
       end         %end of if-else loop
   end    % end of Step 1.3                             
       
   if strcmp(next,'Step1p4')
                                 %Step 1.4 
       p14 = 0;
       for i=1:n
           if J(i) == -1 & abs(A(ell,:)*Dinv(:,i)) >= tol
               p14 =  1;
           end
       end   %end of for loop
       if p14 == 1
           next = 'Step1p5';
       else
       % continue with Step 1.4
           nu = 0;
           big = -inf;
           for i=1:n
               if J(i) == 0
                   testbig = abs(A(ell,:)*Dinv(:,i));
                   if testbig > big
                       big = testbig;
                       nu = i;
                   end
               end
           end     %end of for loop
           Dhatinv = Dinv;
           next = 'Step4p1';
       end   %end of if-else loop
   end    % end of Step 1.4
          
   if strcmp(next,'Step1p5')
                                 %Step 1.5
       nu = 0;
       big = -inf;
       for i=1:n
           if J(i) == -1
               testbig = abs(A(ell,:)*Dinv(:,i));
               if testbig > big
                   big = testbig;
                   nu = i;
               end
           end
       end     %end of for loop
       [Dhatinv] = Phi2(Dinv,A(ell,:)',nu,J,n);
       next = 'Step4p1';
   end    % end of Step 1.5
                                
   if strcmp(next,'Step2')
                                %Step 2
       if A(ell,:)*x > b(ell)
           s = Dinv(:,nu);
       else 
           s = - Dinv(:,nu);
       end
       next = 'Step3';
   end    % end of Step 2
   
   if strcmp(next,'Step3');
                                %Step3
       t1 =  ( A(ell,:)*x - b(ell) );
       t2 = A(ell,:)*s; 
       sighat = abs(t1/t2);
       kay = 0;
       ratio = inf;
       Csj = C*s;
       for i=1:n
           if J(i) == 0
               tau = Dinv(:,i)'*Csj;
               if abs(tau) >= tol
                   test = g' * Dinv(:,i)/tau;
                   if test <= ratio
                       ratio = test;
                       kay = i;
                   end
               end
           ratio
           kay
           elseif J(i) >= 1 & J(i) <= m
               tau = Dinv(:,i)'*Csj;
               vee = g' * Dinv(:,i);
               if tau < -tol
                   test = g' * Dinv(:,i)/tau;
                   if test <= ratio
                       ratio = test;
                       kay = i;
                   end
               end
           end    
       end  % end for i=1:n
       sigtil = ratio;
       sigma = min(sighat,sigtil);
       next = 'Step4p2';
   end    % end of Step 3   
       
                                         
   if strcmp(next,'Step4p1')
                                %Step 4p1
       % no updates required for x, g or fx
       Dinv = Phi1(Dhatinv,A(ell,:)',nu,J,n);
       J(nu) = ell;
       next = 'End';
   end    % end of Step 4.1 
   
   if strcmp(next,'Step4p2')
                                % Step 4p2 :
       x = x - sigma*s;
       g = c + C*x;
       fx = c'*x + 0.5*( x'*C*x);
       if sigma == sighat
           %sigma = sighat: nothing to do
           next = 'End';
       else        %  sigma = sigtil
           d = (Dinv(:,kay)'*C*Dinv(:,kay)).^(-0.5) ...
                                          *C*Dinv(:,kay);
           Dinv = Phi1(Dinv,d,kay,J,n);
           J(kay) = -1;
           next = 'End';
       end
   end    % end of Step 4.2    
   if strcmp(next,'End')
   end    
    
end   %end of for j=0:10000;    
