%    Alg9.m
function [H1,H2,T,IB,fB,HBinv,msg,alg] = ...
         Alg9(c,q,p,C,n,m,tol,xB,IB,nB,A,b,HBinv,tlow,thigh)  
%   Alg9 implements the parametric Simplex Method for 
%   Quadratic Programming.  It uses Procedures Psi1, Psi2 
%   and Psi3 to do the matrix upating.  It is assumed xB, nB, 
%   HBinv and IB contain the optimal solution for t = tlow.
%   Initialization
msg = [];
next = 'Step1';
alg = 'Algorithm 9 (Simplex Method for Parametric QP)';
singular = 'false';
T(1) = tlow;
interval = 0;
for iter=0:10000
    if strcmp(next,'Step1')
                             % Step 1
        [ B,cB,CBB,gB,gN,fB,IN ] = simpobj(A,C,c,n,xB,IB,nB);
        soln1 = HBinv*vertcat(-cB,b);
        qB = q(IB);
        soln2 = HBinv*vertcat(-qB,p);                     
        h1B = soln1(1:nB);
        g1B = soln1(nB+1:nB+m);
        h2B = soln2(1:nB);
        g2B = soln2(nB+1:nB+m);
        w1 = zeros(n,1);
        w2 = zeros(n,1);
        for j=1:n-nB
            i = IN(j);
            sum1 = c(i);
            sum2 = q(i);
            for k=1:nB
                sum1 = sum1 + C(i,IB(k))*h1B(k);
                sum2 = sum2 + C(i,IB(k))*h2B(k);
            end
            sum1 = sum1 + A(:,i)'*g1B;
            sum2 = sum2 + A(:,i)'*g2B;
            w1(i) = sum1;
            w2(i) = sum2;
        end
        interval = interval + 1;
        H1(:,interval)  = zeros(n,1);
        H1(IB,interval) = h1B(:); 
        H2(:,interval)  = zeros(n,1);
        H2(IB,interval) = h2B(:); 

            
        
%                             Interval limits
        that =  Inf;
        ell = 0;
        for i=1:nB
            if h2B(i) <= - tol
                temp = - h1B(i)/h2B(i);
                if temp < that
                    that = temp;
                    ell = i;
                end
            end
        end
        ttilde = Inf;
        kay = 0;
        for i=1:n
            if w2(i) <= - tol
                temp = - w1(i)/w2(i);
                if temp <= ttilde
                    ttilde = temp;
                    kay = i;
                end
            end
        end
        t2 = min(that,ttilde);
        t2 = min(t2,thigh);
        T(interval+1) = t2;
        if t2 == thigh
            return
        end    
        next = 'Step2';    
    end                      % end of Step 1
    
    if strcmp(next,'Step2')
                             % Step 2
        if t2 == that
            next = 'Step2p1';
        else 
            next = 'Step2p2';
        end
    end
    if strcmp(next,'Step2p1')
        [HBinvtemp,singular,w] = Psi2(HBinv,m+nB,ell);
        if strcmp(singular,'false')
            next = 'Step2p1p1';
        else 
            next = 'Step2p1p2';
        end 
    end    
    if strcmp(next,'Step2p1p1');
        HBinv = HBinvtemp;
        IB(ell) = [];
        xB(ell) = [];
        nB = nB - 1;
        next = 'Done';
    end
    if strcmp(next,'Step2p1p2')
        for i=1:m
            xi(i) = -w(nB-1+i);
        end
        kay = 0;
        xmin = Inf;
        for i=1:n-nB
            temp1 = A(:,i)'*xi';
            if temp1 >= tol
                test = (w1(i) + t2*w2(i))/temp1;
                if test <  xmin
                    xmin = test;
                    kay = i;
                end
            end
        end
        if kay == 0
            msg = 'No feasible solution for further t';
            return
        end
        d = [];
        for i=1:ell-1
            d(i) = C(IB(i),kay) - C(IB(i),IB(ell));
        end
        d(ell) = (C(kay,kay) - C(IB(ell),IB(ell)))/2.;
        for i=ell+1:nB
            d(i) = C(IB(i),kay) - C(IB(i),IB(ell));
        end
        for i=1:m
            d(i+nB) = A(i,kay) - A(i,IB(ell));
        end
        d  = d';
        [ HBinv,singular,w ] = ... 
                       Psi3( HBinv,d,nB+m,ell );
        IB(ell) = kay;
        next = 'Done';
    end
    if strcmp(next,'Step2p2')
        d1 = [];
        for i=1:nB
            d1(i) = C(IB(i),kay);
        end
        d1 = d1';
        [HBinvtemp,singular,w] = ...
                   Psi1(HBinv,d1,C(kay,kay),A(:,kay),m+nB,nB);
        if strcmp(singular,'false')
            next = 'Step2p2p1';
        else
            next = 'Step2p2p2';
        end    
    end
    if strcmp(next,'Step2p2p1')
        HBinv = HBinvtemp;
        IB(nB+1) = kay;
        nB = nB + 1;
        next = 'Done';
    end
    if strcmp(next,'Step2p2p2')
        for i=1:nB
            sB(i) = w(i);
        end
        ell = 0;
        sigB = Inf;
        for i=1:nB
            if sB(i) >= tol
                test = (h1B(i) + t2*h2B(i))/sB(i);
                if test < sigB
                    sigB = test;
                    ell = i;
                end
            end
        end
        if ell == 0
            msg = 'objective is unbounded from below';
            return
        end
        d = [];
        for i=1:ell-1
            d(i) = C(IB(i),kay) - C(IB(i),IB(ell));
        end
        d(ell) = (C(kay,kay) - C(IB(ell),IB(ell)))/2.;
        for i=ell+1:nB
            d(i) = C(IB(i),kay) - C(IB(i),IB(ell));
        end
        for i=1:m
            d(i+nB) = A(i,kay) - A(i,IB(ell));
        end
        d  = d';
        [ HBinv,singular,w ] = ... 
                           Psi3( HBinv,d,nB+m,ell );
        IB(ell) = kay;
        next = 'Done';
    end                
    if strcmp(next,'Done')
        next = 'Step1';
    end    
end                          % end of "for iter=0:10000        
